local isme, istarget = _G.COMBATLOG_OBJECT_AFFILIATION_MINE, _G.COMBATLOG_OBJECT_TARGET;
local bband, mceil = bit.band, math.ceil;

local self_guid = nil;
local damage_table = {nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil};

--Interface\\AddOns\\cFrenzied\\consolai.ttf
local FONT = "Fonts\\FRIZQT__.ttf";
local TIME_SLICE = 5;
local MINIMUM = 0.05;
local DAM_MULTIPLIER = 0.4;

--this should include Gift of Elune if you use it and Wildflesh, eg. 1.15 * 1.2 for both
local GEN_MULTIPLIER = 1.15;

local UnitGUID, UnitClass, UnitHealthMax, select = UnitGUID, UnitClass, UnitHealthMax, select;
local GetCombatRatingBonus, CR_VERSATILITY_DAMAGE_DONE, GetVersatilityBonus = GetCombatRatingBonus, CR_VERSATILITY_DAMAGE_DONE, GetVersatilityBonus;

local cFrenzied = CreateFrame("Frame", "cFrenzied", UIParent);
cFrenzied:SetWidth(150);
cFrenzied:SetHeight(30);
cFrenzied:SetPoint("CENTER", UIParent, "CENTER", 0, -60);
cFrenzied.TimeSinceLastUpdate = 5;

local tick = cFrenzied:CreateFontString(nil, "OVERLAY");
tick:SetAllPoints(cFrenzied);
tick:SetFont(FONT, 21);
tick:SetJustifyH("CENTER");
tick:SetShadowOffset(0.7, -0.7);
tick.l_format = "|cffffff00%0.1f%%|r";

cFrenzied:RegisterEvent("ADDON_LOADED");

cFrenzied:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end);

local function clear_damage(K)
	if(true) then return; end;

	local i = #damage_table;

	if(K > i) then return; end;

	for i = #damage_table, K, -1 do
		damage_table[i] = nil;
	end
end;

local function update()
	local N = #damage_table;
	local time = GetTime();
	local amount = 0;

	for i = 1, N, 1 do
		local o = damage_table[i];

		if o == nil or (time - o[2]) > TIME_SLICE then
			clear_damage(i);
			break;
		else
			amount = amount + o[1];
		end;
	end;

	if amount == 0 then
		tick:SetText("");
	else
		local maxHealth = UnitHealthMax("player");
		local min = maxHealth * MINIMUM;
		amount = amount * DAM_MULTIPLIER;

		if(amount > min) then
			local vers = 1 + ((GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE) + GetVersatilityBonus(CR_VERSATILITY_DAMAGE_DONE)) / 100);

			tick:SetFormattedText(tick.l_format, (amount * GEN_MULTIPLIER * vers * 100) / maxHealth);
		else
			tick:SetText("");
		end
	end;
end;

local function OnUpdate(self, elapsed)
  self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed;

  if(self.TimeSinceLastUpdate > 0.2) then
		update();
		self.TimeSinceLastUpdate = 0;
  end;
end;

function cFrenzied:COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, hideCaster, srcGUID, srcName, srcFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, a1, a2, a3, a4)
	local amount = 0;

	if(destGUID == self_guid) then
		if(event == 'SWING_DAMAGE') then
			amount = a1;
		elseif(event == 'SPELL_DAMAGE' or event == 'SPELL_PERIODIC_DAMAGE' or event == 'RANGE_DAMAGE') then
			amount = a4;
		else
			return;
		end;
	end;

	if(amount > 0) then
		table.insert(damage_table, 1, {amount, GetTime()});
		update();
	end;
end;

function cFrenzied:PLAYER_REGEN_ENABLED()
	cFrenzied:SetScript("OnUpdate", nil);
	cFrenzied:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	clear_damage(1);
	tick:SetText("");
end;

function cFrenzied:PLAYER_REGEN_DISABLED()
	if self_guid == nil then self_guid = UnitGUID("player"); end;
	cFrenzied:SetScript("OnUpdate", OnUpdate);
	cFrenzied:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
end;

function cFrenzied:ADDON_LOADED(v)
	if(v ~= "cFrenzied" or select(2, UnitClass("player")) ~= "DRUID") then return end;

	self_guid = UnitGUID("player");

	cFrenzied:RegisterEvent("PLAYER_REGEN_ENABLED");
	cFrenzied:RegisterEvent("PLAYER_REGEN_DISABLED");
end;